/*
 * parser.c
 *
 * Copyright(c) 1999, Particle
 */

#include <stdlib.h>

#include "scanner.h"
#include "error.h"
#include "parser.h"

#define LOCALLY_ALLOCATED_SCANNER 1

/*
 * match function (for LL(1) parsers)
 *
 * recieves a parser (which contains current
 * token) and a string. If string matches current
 * token, scans in another token, and returns 1, 
 * otherwise returns zero (without scanning a new
 * token.
 */
int match(parser_t* parser,char* s){
	if(!parser)
		error("null pointer parameter to match()");
	if(parser->token && parser->token->token){
		if(strcmp(parser->token->token,s))
			return 0;
		freeToken(parser->token);
	}
	parser->token = readToken(parser->scanner);
	return 1;
}

/*
 * force match (gets rid of current token and gets
 * next one)
 */
void forceMatch(parser_t* parser){
	match(parser,parser->token->token);
}

/*
 * creates a parser given a scanner. The parser
 * doesn't take responsibility for the scanner
 * (you must free it yourself)
 */
parser_t* createParser(scanner_t* s){
	parser_t* p = (parser_t*)malloc(sizeof(parser_t));
	if(!p)
		error("cannot allocate %d bytes for parser",sizeof(parser_t));
	p->scanner = s;
	p->token = readToken(p->scanner);
	p->flags = 0;
	return p;
}

/*
 * creates a parser from a file name. The localy
 * allocated scanner is freed automatically upon
 * destruction of this parser.
 */
parser_t* createParserFromFile(char* filename){
	parser_t* p = 0;
	scanner_t* s = 0;
	s = createScannerFromFile(filename);
	p = createParser(s);
	p->flags |= LOCALLY_ALLOCATED_SCANNER;
	return p;
}

/*
 * frees a parser (and all related memory)
 *  uses the flags field of the parser to 
 * determine whether the scanner is locally
 * allocated or not.
 */
void freeParser(parser_t* p){
	if(!p)
		error("null pointer parameter to freeParser()");
	if(p->flags & LOCALLY_ALLOCATED_SCANNER)
		freeScanner(p->scanner);
	if(p->token)
		freeToken(p->token);
	free(p);
}

