/*
 * scanner.c
 *
 * Copyright(c) 1999, Particle
 */

#include <stdlib.h>

#include "scanner.h"
#include "error.h"
#include "files.h"

#define SCANNER_LOCAL_FILE 1

/*
 * creates a canner given a buffer and a length.
 * upon distruction of this canner, the library doesn't
 * free this buffer (in case it is used for other functions.
 */
scanner_t* createScanner(char* buffer,int length){
	scanner_t* p = (scanner_t*)malloc(sizeof(scanner_t));
	if(!p)
		error("cannot allocate %d bytes for scanner",sizeof(scanner_t));
	p->buffer = buffer;
	p->where = 0;
	p->length = length;
	p->flags = 0;
	p->lineno = 1;
	return p;
}

/*
 * creates a canner given a filename. the scanner
 * returned by this function will free the all allocated
 * memory upon distruction.
 */
scanner_t* createScannerFromFile(char* filename){
	char* buffer;
	int length;
	scanner_t* p;
	length = fileLoad(filename,&buffer);
	p = createScanner(buffer,length);
	p->flags |= SCANNER_LOCAL_FILE;
	return p;
}

/*
 * resets scanner to it's initial condition.
 */
void resetScanner(scanner_t* p){
	if(!p)
		error("null pointer parameter to resetScanner");
	p->where = 0;
	p->lineno = 1;
}

/*
 * free scanner. (and all related memory items)
 * this function must be called to ensure that all
 * memory has been freed. (simply calling free() on
 * the scanner is not recommended.
 */
void freeScanner(scanner_t* p){
	if(!p)
		error("null pointer parameter to freeScanner");
	if(p->flags & SCANNER_LOCAL_FILE)
		free(p->buffer);
	free(p);
}

/*
 * make a token given test and beginning and end
 */
token_t* makeToken(char* s,int start,int end){
	token_t* p = (token_t*)malloc(sizeof(token_t));
	if(!p)
		error("cannot allocate %d bytes for token",sizeof(token_t));
	p->length = end-start+1;
	p->token = (char*)malloc(p->length + 1);
	p->u.negative = 0;

	if(!(p->token))
		error("cannot allocate %d bytes for token text",p->length+1);
	memcpy(p->token,&s[start],p->length);
	p->token[p->length] = '\0';

	return p;
}

/*
 * makes a token from a token (copy a token)
 */
token_t* makeTokenFromToken(token_t* t){
	token_t* tmp = makeToken(t->token,0,t->length - 1);
	tmp->u.negative = t->u.negative;
	return tmp;
}

/*
 * free token
 */
void freeToken(token_t* p){
	if(!p)
		error("null pointer parameter to freeToken");
	free(p->token);
	free(p);
}
